from cimlab.loaders.sparql.rc4_2021.get_class_type import get_class_type_sparql
#a
import cimlab.loaders.sparql.rc4_2021.ac_dc_terminal as ACDCTerminalSPARQL
import cimlab.loaders.sparql.rc4_2021.ac_line_segment as ACLineSegmentSPARQL
import cimlab.loaders.sparql.rc4_2021.ac_line_segment_phase as ACLineSegmentPhaseSPARQL
import cimlab.loaders.sparql.rc4_2021.analog as AnalogSPARQL
import cimlab.loaders.sparql.rc4_2021.asset as AssetSPARQL
import cimlab.loaders.sparql.rc4_2021.asset_info as AssetInfoSPARQL
#b
import cimlab.loaders.sparql.rc4_2021.base_voltage as BaseVoltageSPARQL
import cimlab.loaders.sparql.rc4_2021.battery_unit as BatteryUnitSPARQL
import cimlab.loaders.sparql.rc4_2021.breaker as BreakerSPARQL
# import cimlab.loaders.sparql.rc4_2021.bus_bar_section as BusBarSectionSPARQL
# import cimlab.loaders.sparql.rc4_2021.bus_bar_section_info as BusBarSectionInfoSPARQL
#c
import cimlab.loaders.sparql.rc4_2021.cable_info as CableInfoSPARQL
import cimlab.loaders.sparql.rc4_2021.concentric_neutral_cable_info as ConcentricNeutralCableInfoSPARQL
#d
import cimlab.loaders.sparql.rc4_2021.disconnector as DisconnectorSPARQL
import cimlab.loaders.sparql.rc4_2021.discrete as DiscreteSPARQL
#e
import cimlab.loaders.sparql.rc4_2021.energy_consumer as EnergyConsumerSPARQL
import cimlab.loaders.sparql.rc4_2021.energy_consumer_phase as EnergyConsumerPhaseSPARQL
#f
import cimlab.loaders.sparql.rc4_2021.fuse as FuseSPARQL

#l
import cimlab.loaders.sparql.rc4_2021.linear_shunt_compensator as LinearShuntCompensatorSPARQL
import cimlab.loaders.sparql.rc4_2021.load_break_switch as LoadBreakSwitchSPARQL

#n
import cimlab.loaders.sparql.rc4_2021.no_load_test as NoLoadTestSPARQL
#o
import cimlab.loaders.sparql.rc4_2021.overhead_wire_info as OverheadWireInfoSPARQL

#p
import cimlab.loaders.sparql.rc4_2021.per_length_phase_impedance as PerLengthPhaseImpedanceSPARQL
import cimlab.loaders.sparql.rc4_2021.per_length_sequence_impedance as PerLengthSequenceImpedanceSPARQL
import cimlab.loaders.sparql.rc4_2021.phase_impedance_data as PhaseImpedanceDataSPARQL
import cimlab.loaders.sparql.rc4_2021.power_electronics_connection as PowerElectronicsConnectionSPARQL
import cimlab.loaders.sparql.rc4_2021.power_electronics_connection_phase as PowerElectronicsConnectionPhaseSPARQL
import cimlab.loaders.sparql.rc4_2021.power_transformer as PowerTransformerSPARQL
import cimlab.loaders.sparql.rc4_2021.power_transformer_end as PowerTransformerEndSPARQL
import cimlab.loaders.sparql.rc4_2021.power_transformer_info as PowerTransformerInfoSPARQL
#r
import cimlab.loaders.sparql.rc4_2021.ratio_tap_changer as RatioTapChangerSPARQL
import cimlab.loaders.sparql.rc4_2021.recloser as RecloserSPARQL
#s
import cimlab.loaders.sparql.rc4_2021.sectionaliser as SectionaliserSPARQL
import cimlab.loaders.sparql.rc4_2021.short_circuit_test as ShortCircuitTestSPARQL
import cimlab.loaders.sparql.rc4_2021.synchronous_machine as SynchronousMachineSPARQL
import cimlab.loaders.sparql.rc4_2021.switch_phase as SwitchPhaseSPARQL
#t
import cimlab.loaders.sparql.rc4_2021.tap_changer_control as TapChangerControlSPARQL
import cimlab.loaders.sparql.rc4_2021.tape_shield_cable_info as TapeShieldCableInfoSPARQL
import cimlab.loaders.sparql.rc4_2021.terminal as TerminalSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_end_info as TransformerEndInfoSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_tank as TransformerTankSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_tank_end as TransformerTankEndSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_tank_info as TransformerTankInfoSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_mesh_impedance as TransformerMeshImpedanceSPARQL
import cimlab.loaders.sparql.rc4_2021.transformer_core_admittance as TransformerCoreAdmittanceSPARQL

#w
import cimlab.loaders.sparql.rc4_2021.wire_position as WirePositionSPARQL
import cimlab.loaders.sparql.rc4_2021.wire_spacing_info as WireSpacingInfoSPARQL


from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        sparql_message: query string that can be used in blazegraph connection or STOMP client
    """
    
    mrid_list = list(typed_catalog[cim.TapeShieldCableInfo].keys())
    asset_list = list(typed_catalog[cim.ACLineSegment].keys())
                      
    query_message = """
        PREFIX r: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim: <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?coreRadius ?coreStrandCount ?gmr ?insulated ?insulationMaterial 
         ?insulationThickness ?material ?rAC25 ?rAC50 ?rAC75 ?rDC20 ?radius ?ratedCurrent ?strandCount
         ?sizeDescription ?constructionKind ?diameterOverInsulation ?diameterOverJacket ?diameterOverScreen
         ?isStrandFill ?nominalTemperature ?sheathAsNeutral ?shieldMaterial ?tapeLap ?tapeThickness
         (group_concat(distinct ?ACLineSegmentPhase; separator=';') as ?ACLineSegmentPhases)
        WHERE {          
          ?eq r:type cim:TapeShieldCableInfo.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {""" % feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n' % mrid
        
    # add all assets
    query_message += """               }
        VALUES ?ACLineSegment {"""
    for asset_mrid in asset_list:
        query_message += ' "%s" \n' % asset_mrid
        
    # add all attributes
    query_message += """               } 
        #trace back up to EquipmentContainer via ACLineSegment
        ?fdr cim:IdentifiedObject.mRID ?fdrid.
        ?line cim:Equipment.EquipmentContainer ?fdr.
        ?linephase cim:ACLineSegmentPhase.ACLineSegment ?line.
        ?line cim:IdentifiedObject.mRID ?ACLineSegment.
        ?linephase cim:ACLineSegmentPhase.WireInfo ?eq.
        ?linephase cim:IdentifiedObject.mRID ?ACLineSegmentPhase.

        ?eq cim:IdentifiedObject.mRID ?mRID.
        ?eq cim:IdentifiedObject.name ?name.
        
        #collect inherited WireInfo attributes
        OPTIONAL {?eq cim:WireInfo.coreRadius ?coreRadius.}
        OPTIONAL {?eq cim:WireInfo.coreStrandCount ?coreStrandCount.}        
        OPTIONAL {?eq cim:WireInfo.gmr ?gmr.}
        OPTIONAL {?eq cim:WireInfo.insulated ?insulated.} 
        OPTIONAL {?eq cim:WireInfo.insulationMaterial ?ins.
                  bind(strafter(str(?ins),"WireInsulationKind.") as ?insulationMaterial).}
        OPTIONAL {?eq cim:WireInfo.insulationThickness ?insulationThickness.}
        OPTIONAL {?eq cim:WireInfo.material ?material.}
        OPTIONAL {?eq cim:WireInfo.rAC25 ?rAC25.}
        OPTIONAL {?eq cim:WireInfo.rAC50 ?rAC50.}
        OPTIONAL {?eq cim:WireInfo.rAC75 ?rAC75.}
        OPTIONAL {?eq cim:WireInfo.rDC20 ?rDC20.}
        OPTIONAL {?eq cim:WireInfo.radius ?radius.}
        OPTIONAL {?eq cim:WireInfo.ratedCurrent ?ratedCurrent.}
        OPTIONAL {?eq cim:WireInfo.strandCount ?strandCount.}
        OPTIONAL {?eq cim:WireInfo.sizeDescription ?sizeDescription.}

        #collect inherited CableInfo attributes
        OPTIONAL {?eq cim:CableInfo.constructionKind ?constructionKind.}
        OPTIONAL {?eq cim:CableInfo.diameterOverCore ?diameterOverCore.}
        OPTIONAL {?eq cim:CableInfo.diameterOverInsulation ?diameterOverInsulation.}
        OPTIONAL {?eq cim:CableInfo.diameterOverJacket ?diameterOverJacket.}
        OPTIONAL {?eq cim:CableInfo.diameterOverScreen ?diameterOverScreen.}
        OPTIONAL {?eq cim:CableInfo.isStrandFill ?isStrandFill.}
        OPTIONAL {?eq cim:CableInfo.nominalTemperature ?nominalTemperature.}
        OPTIONAL {?eq cim:CableInfo.outerJacketKind ?outerJacketKind.}
        OPTIONAL {?eq cim:CableInfo.sheathAsNeutral ?sheathAsNeutral.}
        OPTIONAL {?eq cim:CableInfo.shieldMaterial ?shieldMaterial.}
        
        #collect TapeShieldCableInfo attributes
        OPTIONAL {?eq cim:TapeShieldCableInfo.tapeLap ?tapeLap.}
        OPTIONAL {?eq cim:TapeShieldCableInfo.tapeThickness ?tapeThickness.}

        }
        GROUP BY ?mRID ?name ?coreRadius ?coreStrandCount ?gmr ?insulated ?insulationMaterial 
         ?insulationThickness ?material ?rAC25 ?rAC50 ?rAC75 ?rDC20 ?radius ?ratedCurrent ?strandCount
         ?sizeDescription ?constructionKind ?diameterOverInsulation ?diameterOverJacket ?diameterOverScreen
         ?isStrandFill ?nominalTemperature ?sheathAsNeutral ?shieldMaterial ?tapeLap ?tapeThickness
         
        ORDER by  ?name
        """
    return query_message


from __future__ import annotations
from typing import Dict, List, Optional
from SPARQLWrapper import SPARQLWrapper, JSON, POST
import re
import importlib
from cimlab.loaders import ConnectionInterface, ConnectionParameters, Parameter, QueryResponse
from cimlab.models.model_parsers import add_to_catalog, add_to_typed_catalog


class BlazegraphConnection(ConnectionInterface):
    def __init__(self, connection_params, cim_profile:str):
        self.sparql = importlib.import_module('cimlab.loaders.sparql.' + cim_profile)
        self.cim = importlib.import_module('cimlab.data_profile.' + cim_profile)
        self.sparql_obj: Optional[SPARQLWrapper] = None
        self.connection_params = connection_params

    def connect(self):
        if not self.sparql_obj:
            url = self.connection_params.parameters[0].value
            self.sparql_obj = SPARQLWrapper(url)
            self.sparql_obj.setReturnFormat(JSON)

    def disconnect(self):
        self.sparql_obj = None
        
    def execute(self, query_message: str) -> QueryResponse:
        self.connect()
        self.sparql_obj.setQuery(query_message)
        self.sparql_obj.setMethod(POST)
        query_output = self.sparql_obj.query().convert()
        return query_output

        
    def create_default_instances(self, feeder_mrid: str | cim.Feeder, mrid_list: List[str]) -> List[object]:
        """ 
        Creates empty CIM objects with the correct class type with mRID and name fields populated based on 
        a list of mRID strings.
        Args:
            feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
            mrid_list (list[str]): A list of object mRID strings to be converted into CIM objects
        Returns:
            object_list: A list of CIM object instances
        """
        #generate correct sparql message using create_default.py
        sparql_message = self.sparql.get_class_type_sparql(feeder_mrid, mrid_list)
        #execute sparql query
        query_output = self.execute(sparql_message)
        # parse query results and add new CIM objects to list
        object_list = [] 
        for result in query_output['results']['bindings']:
           # print(result)
            cls = result['class']['value']
            mRID = result['mRID']['value']
            name = result['name']['value']
            try:
                object_list.append(eval(f"self.cim.{cls}(mRID='{mRID}', name = '{name}')"))
            except:
                print('warning: object class missing from data profile:', cls)
        return object_list
    
    
          
    def get_all_attributes(self, feeder_mrid: str | cim.Feeder, typed_catalog: dict[type, dict[str, object]], cim_class: type):
        """ Populates all available attribute fields of CIM objects in the typed catalog of a specified CIM class. 
        Objects are stored in memory, so no values are returned.
        Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
        cim_class (type): The CIM class type (e.g. cim:ACLineSegment)
        Returns:
        none
        """
        #generate SPARQL message from correct loaders>sparql python script based on class name
        sparql_message = self.get_attributes_query(feeder_mrid, typed_catalog, cim_class)
        #execute sparql query
#         print(sparql_message)
        query_output = self.execute(sparql_message)
#         print(query_output)

#         attribute_list = query_output['head']['vars'] #list of attributes received in query response
        for result in query_output['results']['bindings']: #iterate through rows of response
            attribute_list = result.keys()
            mRID = result['mRID']['value']
            for attribute in attribute_list: 
                try: #check if attribute is in data profile
                    attribute_type = cim_class.__dataclass_fields__[attribute].type
                except:
                    #replace with warning message
                    print('warning: attribute ', attribute, ' missing from ', cim_class.__name__)
                    
                if 'List' in attribute_type: #check if attribute is association to a list of class objects
                    if '\'' in attribute_type: #handling inconsistent '' marks in data profile
                        at_cls = re.match(r'List\[\'(.*)\']',attribute_type)
                        attribute_class = at_cls.group(1)
                    else:        
                        at_cls = re.match(r'List\[(.*)]',attribute_type)
                        attribute_class = at_cls.group(1)
#                     print(attribute_class)
                    # pass query response of associated objects to list parser
                    self.query_list_parser(feeder_mrid, typed_catalog, cim_class, mRID, result, attribute, attribute_class, ';')
                elif 'Optional' in attribute_type: #check if attribute is association to a class object
                    if '\'' in attribute_type: #handling inconsistent '' marks in data profile
                        at_cls = re.match(r'Optional\[\'(.*)\']',attribute_type)
                        attribute_class = at_cls.group(1)
                    else:        
                        at_cls = re.match(r'Optional\[(.*)]',attribute_type)
                        attribute_class = at_cls.group(1)
#                     print(attribute_class)
                    # pass query response of associated objects to list parser
                    self.query_parser(feeder_mrid, typed_catalog, cim_class, mRID, result, attribute, attribute_class, ';')
                else: #otherwise assign query response
#                     print(attribute)
#                     print(result)
                    self.query_parser(feeder_mrid, typed_catalog, cim_class, mRID, result, attribute, attribute_class, ';')
                    
    def get_attributes_query(self, feeder_mrid: str | cim.Feeder, typed_catalog: dict[type, dict[str, object]], cim_class: type):
        """ Generates SPARQL query for a given catalog of objects and feeder id
        Args:
            feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
            typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
                class type and object mRID
            cim_class (type): The CIM class type (e.g. cim:ACLineSegment)
        Returns:
            sparql_message: query string that can be used in blazegraph connection or STOMP client
        none
        """
#         mrid_list = list(typed_catalog[cim_class].keys()) #get all object mRIDs in switch area
        #generate SPARQL message from correct loaders>sparql python script based on class name
#         sparql_message = eval(f"self.sparql.{cim_class.__name__}SPARQL.get_all_attributes('{feeder_mrid}', {mrid_list})")
        sparql_func = getattr(self.sparql, f"{cim_class.__name__}SPARQL")
        sparql_message = sparql_func.get_all_attributes(feeder_mrid, typed_catalog)
        
        return sparql_message
    
    
    def query_parser(self, feeder_mrid, typed_catalog:Dict, class_name:str, mRID:str, query:List, attribute:str, attribute_class:str, separator:str) -> object | str:
    #     try:
        value = query[attribute]['value']

        #if attribute is CIM class, then build CIM objects. otherwise assign to obj_list
    #         print(attribute)
        if attribute_class in self.cim.__all__:
            value = value.split(separator)
            class_type = eval(f"self.cim.{attribute_class}")
            if type(class_type) is type:
                result = self.build_cim_object(feeder_mrid, typed_catalog, value, attribute_class)  
            else:
                result = [value]
            if len(result) == 1:
                result = result[0]
        else:
            result = value
        setattr(typed_catalog[class_name][mRID], attribute, result)

    def query_list_parser(self, feeder_mrid, typed_catalog:Dict, class_name:type, mRID:str, query:List, attribute:str, attribute_class:str, separator:str):
        value = query[attribute]['value']
        values = value.split(separator)
        #if attribute is CIM class, then build CIM objects. otherwise assign to obj_list
        if attribute_class in self.cim.__all__:
            class_type = eval(f"self.cim.{attribute_class}")
            if type(class_type) is type:
                obj_list = self.build_cim_object(feeder_mrid, typed_catalog, values, attribute_class)
            else:
                obj_list = values
        else:
            obj_list = values
        #set attribute of queried object to list parsed from query results
        setattr(typed_catalog[class_name][mRID], attribute, obj_list)


    def build_cim_object(self, feeder_mrid, typed_catalog:Dict, mRID_list:List[str], default_class = 'IdentifiedObject') -> List(object):
        sparql_message = self.sparql.get_class_type_sparql(feeder_mrid, mRID_list)
        #execute sparql query
        query_output = self.execute(sparql_message)
        # parse query results and add new CIM objects to list
        obj_list = [] 
        for result in query_output['results']['bindings']:
            class_name = result['class']['value']
            mRID = result['mRID']['value']
            name = result['name']['value']
            obj = self.create_object(typed_catalog, class_name, mRID, name)
            obj_list.append(obj)
        
        if '' not in mRID_list and not obj_list:
            for mrid in mRID_list:
                print('warning: could not locate mrid: ', mrid,'. creating default object of ', default_class)
                obj = self.create_object(typed_catalog, default_class, mrid, None)
                obj_list.append(obj)
        return obj_list

    def create_object(self, typed_catalog, class_name, mRID, name):
        
        class_type = eval(f"self.cim.{class_name}")
        #add class to typed_catalog if not already defined
        if class_type not in typed_catalog.keys():
            typed_catalog[class_type] = {}

        if mRID in typed_catalog[class_type].keys():
            obj = typed_catalog[class_type][mRID]
        else:
#                 print(class_type)
                obj = class_type(mRID = mRID, name = name)
                add_to_typed_catalog(obj, typed_catalog)
#         except:
#         obj = mRID
#         print('warning: object class missing from data profile:', cls)
        return obj



from __future__ import annotations
from typing import List
from dataclasses import dataclass, field
import json

import cimlab.data_profile as cim
from cimlab.loaders import ConnectionInterface
from cimlab.models.model_parsers import add_to_catalog, add_to_typed_catalog, cim_dump




@dataclass
class SecondaryArea:
    area_id: str
    connection: ConnectionInterface
    distribution_transformer: dict[str, object] = field(default_factory=dict)
    addressable_equipment: dict[str, object] = field(default_factory=dict)
    unaddressable_equipment: dict[str, object] = field(default_factory=dict)
    connectivity_nodes: dict[str, object] = field(default_factory=dict)
    secondary_areas: list[object] = field(default_factory=list)
    typed_catalog: dict[type, dict[str, object]] = field(default_factory=dict)

    # Initialize empty CIM objects for all equipment in secondary area
    def initialize_secondary_area(self, switch_msg: dict):
        self.feeder_mrid = self.area_id.split('.')[0]  # get feeder mRID from area id

        addr_equip = self.connection.create_default_instances(self.feeder_mrid, switch_msg['addressable_equipment'])
        for obj in addr_equip:
            add_to_catalog(obj, self.addressable_equipment)
            add_to_typed_catalog(obj, self.typed_catalog)
        unaddr_equip = self.connection.create_default_instances(self.feeder_mrid, switch_msg['unaddressable_equipment'])
        for obj in unaddr_equip:
            add_to_catalog(obj, self.unaddressable_equipment)
            add_to_typed_catalog(obj, self.typed_catalog)
        conn_nodes = self.connection.create_default_instances(self.feeder_mrid, switch_msg['connectivity_node'])
        for obj in conn_nodes:
            add_to_catalog(obj, self.connectivity_nodes)
            add_to_typed_catalog(obj, self.typed_catalog)
        xfmr = self.connection.create_default_instances(self.feeder_mrid, switch_msg['distribution_transformer'])
        for obj in xfmr:
            add_to_catalog(obj, self.distribution_transformer)
            add_to_typed_catalog(obj, self.typed_catalog)
            
    def get_all_attributes(self, cim_class):
        if cim_class in self.typed_catalog:
            self.connection.get_all_attributes(self.feeder_mrid, self.typed_catalog, cim_class)
        else:
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')
#             return self.__dumps__(cim_class)

    def get_attributes_query(self, cim_class):
        if cim_class in self.typed_catalog:
            sparql_message = self.connection.get_attributes_query(self.feeder_mrid, self.typed_catalog, cim_class)
        else:
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')
            sparql_message = ''
        return sparql_message

    def __dumps__(self, cim_class):
        if cim_class in self.typed_catalog:
            json_dump = cim_dump(self.typed_catalog, cim_class)
        else:
            json_dump = {}
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')

        return json_dump


from typing import List, Dict
from cimlab.models.distributed_model import DistributedModel
from cimlab.models.switch_area import SwitchArea
from cimlab.models.secondary_area import SecondaryArea
from cimlab.models.model_parsers import add_to_catalog, add_to_typed_catalog, get_all_by_type



from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import json

import cimlab.data_profile as cim
from cimlab.loaders import ConnectionInterface, QueryResponse
from cimlab.models.model_parsers import add_to_catalog, add_to_typed_catalog, cim_dump
from cimlab.models.switch_area import SwitchArea


@dataclass
class DistributedModel:
    feeder: cim.Feeder
    connection: ConnectionInterface
    topology: Dict
    addressable_equipment: Dict[str, object] = field(default_factory=dict)
    unaddressable_equipment: Dict[str, object] = field(default_factory=dict)
    connectivity_nodes: Dict[str, object] = field(default_factory=dict)
    switch_areas: List[SwitchArea] = field(default_factory=list)
    assets: List[str] = field(default_factory=list)
    typed_catalog: dict[type, dict[str, object]] = field(default_factory=dict)
    is_loaded: Optional[set] = field(default_factory=set)

    def __post_init__(self):
        self.__initialize_network__()

    # REPLACE WITH CONNECTION OBJECT



    # Initialize all CIM objects in feeder model
    def __initialize_network__(self) -> Dict[str, object]:
        # Get switch area message from Topology Processor
        # topo_message = DistributedModel.get_topology_api_response(feeder_mrid)
        # topo_message = DistributedModel.get_topology_response(feeder_mrid)

        # Initialize all CIM objects not contained in a switch area
        addr_equip = self.connection.create_default_instances(self.feeder.mRID, self.topology['addressable_equipment'])
        for obj in addr_equip:
            add_to_catalog(obj, self.addressable_equipment)
            add_to_typed_catalog(obj, self.typed_catalog)
        unaddr_equip = self.connection.create_default_instances(self.feeder.mRID,
                                                                self.topology['unaddressable_equipment'])
        for obj in unaddr_equip:
            add_to_catalog(obj, self.unaddressable_equipment)
            add_to_typed_catalog(obj, self.typed_catalog)
        conn_nodes = self.connection.create_default_instances(self.feeder.mRID,
                                                              self.topology['connectivity_node'])
        for obj in conn_nodes:
            add_to_catalog(obj, self.connectivity_nodes)
            add_to_typed_catalog(obj, self.typed_catalog)

        # Initialize all CIM objects in a single switch area
        # def initialize_switch_areas(feeder_mrid) -> dict(str,object):
        sa_index = -1
        for switch_msg in self.topology['switch_areas']:
            # Add switch area
            switch_area = SwitchArea(self.feeder.mRID + '.' + str(sa_index), self.connection)
            switch_area.initialize_switch_area(switch_msg)
            self.switch_areas.append(switch_area)
            # Add switch area unaddressable equipment to feeder unaddressable equipment
            self.unaddressable_equipment.update(switch_area.unaddressable_equipment)
            sa_index = sa_index + 1


    def get_all_attributes(self, cim_class):
        if cim_class in self.typed_catalog:
            self.connection.get_all_attributes(self.feeder.mRID, self.typed_catalog, cim_class)
        else:
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')
#             return self.__dumps__(cim_class)

    def get_attributes_query(self, cim_class):
        if cim_class in self.typed_catalog:
            sparql_message = self.connection.get_attributes_query(self.feeder.mRID, self.typed_catalog, cim_class)
        else:
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')
            sparql_message = ''
        return sparql_message

    def __dumps__(self, cim_class):
        if cim_class in self.typed_catalog:
            json_dump = cim_dump(self.typed_catalog, cim_class)
        else:
            json_dump = {}
            print('warning: no instances of ', cim_class.__name__, ' found in catalog.')

        return json_dump
    
    


from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        query_message: query string that can be used in blazegraph connection or STOMP client
    """
    
    mrid_list = list(typed_catalog[cim.PhotovoltaicUnit].keys())
    asset_list = list(typed_catalog[cim.PowerElectronicsConnection].keys())
    
    query_message = """
        PREFIX r:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim:  <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?maxP ?minP ?inService ?PowerElectronicsConnection
        (group_concat(distinct ?Measurement; separator=";") as ?Measurements) 
 
        WHERE {          
          ?eq r:type cim:PowerElectronicsConnection.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {"""%feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n'%mrid
        
    # add all assets
    query_message += """               }
        VALUES ?PowerElectronicsConnection {"""
    for asset_mrid in asset_list:
        query_message += ' "%s" \n' % asset_mrid
        
    # add all attributes
    query_message += """               } 
        ?pec cim:PowerElectronicsConnection.PowerElectronicsUnit ?eq.
        ?pec cim:IdentifiedObject.mRID ?PowerElectronicsConnection.
        ?pec cim:Equipment.EquipmentContainer ?fdr.
        ?fdr cim:IdentifiedObject.mRID ?fdrid.
        
        ?eq cim:IdentifiedObject.mRID ?mRID.
        ?eq cim:IdentifiedObject.name ?name.

        OPTIONAL {?meas cim:Measurement.PowerSystemResource ?eq.
          ?meas cim:IdentifiedObject.mRID ?Measurement}

        OPTIONAL {?eq cim:PowerElectronicsUnit.maxP ?maxP.}
        OPTIONAL {?eq cim:PowerElectronicsUnit.minP ?maxQ.}
        
        #OPTIONAL {?eq cim:Equipment.aggregate ?aggregate.}
        OPTIONAL {?eq cim:Equipment.inService ?inService.}
        #OPTIONAL {?eq cim:Equipment.networkAnalysisEnable ?networkAnalysisEnabled.}
        #OPTIONAL {?eq cim:Equipment.normallyInService ?normallyInService.}

                  
        }
        GROUP by ?mRID ?name ?maxP ?minP ?inService ?PowerElectronicsConnection
        ORDER by  ?name
        """
    return query_message


from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ 
    Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        query_message: query string that can be used in blazegraph connection or STOMP client
    """

    mrid_list = list(typed_catalog[cim.Analog].keys())


    query_message = """
        PREFIX r:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim:  <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?PowerSystemResource ?measurementType ?phases ?Terminal 
        ?minValue ?maxValue ?normalValue ?positiveFlowIn
        (group_concat(distinct ?Location; separator=';') as ?Locations)

        WHERE {          
          ?eq r:type cim:Analog.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {"""%feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n'%mrid
    # add all attributes
    query_message += """               } 
        ?eq cim:Measurement.PowerSystemResource ?psr.
        ?psr cim:Equipment.EquipmentContainer ?fdr.
        ?fdr cim:IdentifiedObject.mRID ?fdrid.
        ?eq cim:IdentifiedObject.mRID ?mRID.
        ?eq cim:IdentifiedObject.name ?name.
        
        ?eq cim:Measurement.Terminal ?t.
        ?t cim:IdentifiedObject.mRID ?Terminal.

        OPTIONAL {?eq cim:Measurement.phases ?phs.
                 bind(strafter(str(?phs),"PhaseCode.") as ?phases)}
                 
        OPTIONAL {?psr cim:PowerSystemResource.Location ?loc.
                  ?loc cim:IdentifiedObject.mRID ?Location.}
        
        OPTIONAL {?eq cim:Measurement.measurementType ?measurementType.}
        OPTIONAL {?eq cim:Measurement.minValue ?minValue.}
        OPTIONAL {?eq cim:Measurement.maxValue ?maxValue.}
        OPTIONAL {?eq cim:Measurement.normalValue ?normalValue.}
        OPTIONAL {?eq cim:Measurement.positiveFlowIn ?postiveFlowIn.}
        # AnalogValues
        # LimitSets
        # Procedures

       

        }
        GROUP by ?mRID ?name ?PowerSystemResource ?Location ?measurementType ?phases ?Terminal 
        ?minValue ?maxValue ?normalValue ?positiveFlowIn
        
        ORDER by  ?name
        """
    return query_message


from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        query_message: query string that can be used in blazegraph connection or STOMP client
    """

    mrid_list = list(typed_catalog[cim.PhaseImpedanceData].keys())
    asset_list = list(typed_catalog[cim.ACLineSegment].keys())
    
    query_message = """
        PREFIX r:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim:  <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?PhaseImpedance ?row ?column ?r ?x ?b 

        WHERE {          
          ?eq r:type cim:PhaseImpedanceData.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {"""%feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n'%mrid
        
    # add all assets
    query_message += """               }
        VALUES ?ACLineSegment {"""
    for asset_mrid in asset_list:
        query_message += ' "%s" \n' % asset_mrid
        
    # add all attributes
    query_message += """               } 
        ?eq cim:PhaseImpedanceData.PhaseImpedance ?pli.
        ?line cim:ACLineSegment.PerLengthImpedance ?pli.
        ?pli cim:IdentifiedObject.mRID ?PhaseImpedance.
        ?line cim:Equipment.EquipmentContainer ?fdr.
        ?line cim:IdentifiedObject.mRID ?ACLineSegment.
        ?fdr cim:IdentifiedObject.mRID ?fdrid.
        #?eq c:IdentifiedObject.mRID ?mRID. #mRID MISSING FROM XML
        bind(strafter(str(?eq),"sparql#") as ?mRID).
        #?eq cim:IdentifiedObject.name ?name.

        OPTIONAL {?eq cim:PhaseImpedanceData.row ?row.}
        OPTIONAL {?eq cim:PhaseImpedanceData.column ?column.}
        OPTIONAL {?eq cim:PhaseImpedanceData.r ?r.}
        OPTIONAL {?eq cim:PhaseImpedanceData.x ?x.}
        OPTIONAL {?eq cim:PhaseImpedanceData.b ?b.}

        }
        GROUP by ?mRID ?name ?PhaseImpedance ?row ?column ?r ?x ?b 
        ORDER by  ?name
        """
    return query_message


from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        query_message: query string that can be used in blazegraph connection or STOMP client
    """

    mrid_list = list(typed_catalog[cim.TransformerTankEnd].keys())
    asset_list = list(typed_catalog[cim.TransformerTank].keys())
    
    query_message = """
        PREFIX r:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim:  <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?TransformerTank ?phases ?endNumber
        ?grounded ?rground ?xground ?Terminal ?BaseVoltage ?PhaseTapChanger ?RatioTapChanger
        ?FromMeshImpedance ?ToMeshImpedance ?CoreAdmittance
        WHERE {          
          ?eq r:type cim:TransformerTankEnd.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {"""%feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n'%mrid
        
    # add all assets
    query_message += """               }
        VALUES ?TransformerTank {"""
    for asset_mrid in asset_list:
        query_message += ' "%s" \n' % asset_mrid
        
    # add all attributes
    query_message += """          }
        #get feeder id from TransformerTank
        ?eq cim:TransformerTankEnd.TransformerTank ?tank.
        ?tank cim:IdentifiedObject.mRID ?TransformerTank.
        ?tank cim:Equipment.EquipmentContainer ?fdr.
        ?fdr cim:IdentifiedObject.mRID ?fdrid.

        ?eq cim:IdentifiedObject.mRID ?mRID.
        ?eq cim:IdentifiedObject.name ?name.
        
        ?tank cim:IdentifiedObject.mRID ?TransformerTank.

        OPTIONAL {?eq cim:TransformerTankEnd.phases ?phs.
                  bind(strafter(str(?conn),"#PhaseCode.") as ?phases).}

        OPTIONAL {?eq cim:TransformerEnd.endNumber ?endNumber.}
        OPTIONAL {?eq cim:TransformerEnd.grounded ?grounded.}
        OPTIONAL {?eq cim:TransformerEnd.rground ?rground.}
        OPTIONAL {?eq cim:TransformerEnd.xground ?xground.}
        OPTIONAL {?eq cim:TransformerEnd.Terminal ?t.
                  ?t cim:IdentifiedObject.mRID ?Terminal.}
        OPTIONAL {?eq cim:TransformerEnd.BaseVoltage ?bv.
                  ?bv cim:IdentifiedObject.mRID ?BaseVoltage.}
        
        OPTIONAL {?ptc cim:PhaseTapChanger.TransformerEnd ?eq.
                  ?ptc cim:IdentifiedObject.mRID ?PhaseTapChanger.}
        
        OPTIONAL {?rtc cim:RatioTapChanger.TransformerEnd ?eq.
                  ?rtc cim:IdentifiedObject.mRID ?RatioTapChanger.}
                  
        OPTIONAL {?tac cim:TransformerCoreAdmittace.TransformerEnd ?eq.
                  ?tac cim:IdentifiedObject.mRID ?CoreAdmittance.}
          
        OPTIONAL {?frmesh cim:TransformerMeshImpedance.FromTransformerEnd ?eq.
                  ?frmesh cim:IdentifiedObject.mRID ?FromMeshImpedance.}
          
        OPTIONAL {?frmesh cim:TransformerMeshImpedance.ToTransformerEnd ?eq.
                  ?frmesh cim:IdentifiedObject.mRID ?ToMeshImpedance.}
        }
        GROUP by ?mRID ?name ?TransformerTank ?phases ?endNumber
        ?grounded ?rground ?xground ?Terminal ?BaseVoltage ?PhaseTapChanger ?RatioTapChanger
        ?FromMeshImpedance ?ToMeshImpedance ?CoreAdmittance
        ORDER by ?name
          """
    return query_message


from cimlab.data_profile.rc4_2021.gridappsd_cim_profile_rc4_2021 import (
    ACDCTerminal,
    ACLineSegment,
    ACLineSegmentPhase,
    AcceptanceTest,
    AccountNotification,
    AccumulationKind,
    Accumulator,
    AccumulatorLimit,
    AccumulatorLimitSet,
    AccumulatorReset,
    AccumulatorValue,
    ActivePowerLimit,
    ActivityRecord,
    AggregateKind,
    AggregateScore,
    Agreement,
    AirCompressor,
    AlertTypeList,
    AmiBillingReadyKind,
    Analog,
    AnalogControl,
    AnalogLimit,
    AnalogLimitSet,
    AnalogValue,
    Analytic,
    AnalyticKind,
    AnalyticScore,
    ApparentPowerLimit,
    Appointment,
    Approver,
    Asset,
    AssetContainer,
    AssetDeployment,
    AssetFailureTypeification,
    AssetFailureMode,
    AssetFunction,
    AssetGroup,
    AssetGroupKind,
    AssetHazardKind,
    AssetHealthEvent,
    AssetInfo,
    AssetKind,
    AssetLocationHazard,
    AssetModelUsageKind,
    AssetOrganisationRole,
    AssetOwner,
    AssetTestLab,
    AssetTestSampleTaker,
    AssetUser,
    AsynchronousMachine,
    AsynchronousMachineKind,
    AtmosphericAnalog,
    AtmosphericAnalogKind,
    AtmosphericPhenomenon,
    Author,
    BaseFrequency,
    BasePower,
    BaseReading,
    BaseVoltage,
    BasicIntervalSchedule,
    BatteryState,
    BatteryUnit,
    Bay,
    BranchGroup,
    BranchGroupTerminal,
    Breaker,
    BreakerApplicationKind,
    BreakerConfiguration,
    BreakerFailureReasonKind,
    BusNameMarker,
    BusbarConfiguration,
    BusbarSection,
    BusbarSectionInfo,
    Bushing,
    BushingInfo,
    BushingInsulationKind,
    CAESPlant,
    CIM_GridAPPS_D_RC4_2021,
    Cabinet,
    CableConstructionKind,
    CableInfo,
    CableOuterJacketKind,
    CableShieldMaterialKind,
    CatalogAssetType,
    Channel,
    Clamp,
    TypeificationCondition,
    CloudCondition,
    CloudKind,
    CogenerationPlant,
    ComDirectionKind,
    ComFunction,
    ComMedia,
    ComModule,
    ComTechnologyKind,
    CombinedCyclePlant,
    Command,
    CommodityKind,
    CompositeSwitch,
    ConcentricNeutralCableInfo,
    ConductingEquipment,
    Conductor,
    ConfigurationEvent,
    ConformLoad,
    ConformLoadGroup,
    ConformLoadSchedule,
    ConnectivityNode,
    ConnectivityNodeContainer,
    Connector,
    Control,
    ControlledAppliance,
    CoordinateSystem,
    CorporateStandardKind,
    CoverageCodeKind,
    Crew,
    CrewMember,
    CrewStatusKind,
    CrewType,
    Currency,
    CurrentLimit,
    Curve,
    CurveData,
    CurveStyle,
    Customer,
    CustomerAccount,
    CustomerAgreement,
    CustomerKind,
    CustomerNotification,
    Cut,
    Cyclone,
    DERCurveData,
    DERFunction,
    DERGroupDispatch,
    DERGroupForecast,
    DERMonitorableParameter,
    DERParameterKind,
    DERUnitSymbol,
    DateInterval,
    DateTimeInterval,
    DayType,
    DecimalQuantity,
    DemandResponseProgram,
    DeploymentDate,
    DeploymentStateKind,
    DiagnosisDataSet,
    Disconnector,
    Discrete,
    DiscreteCommand,
    DiscreteValue,
    DispatchSchedule,
    DispatchablePowerCapability,
    Document,
    DocumentPersonRole,
    DuctBank,
    EarthFaultCompensator,
    Earthquake,
    Editor,
    ElectronicAddress,
    EmissionAccount,
    EmissionCurve,
    EmissionType,
    EmissionValueSource,
    EndDevice,
    EndDeviceAction,
    EndDeviceCapability,
    EndDeviceControl,
    EndDeviceControlType,
    EndDeviceEvent,
    EndDeviceEventDetail,
    EndDeviceEventType,
    EndDeviceFunction,
    EndDeviceFunctionKind,
    EndDeviceGroup,
    EndDeviceInfo,
    EndDeviceTiming,
    EnergyArea,
    EnergyConnection,
    EnergyConsumer,
    EnergyConsumerPhase,
    EnergySource,
    EnergySourcePhase,
    EnvironmentalAlert,
    EnvironmentalAnalog,
    EnvironmentalCodedValue,
    EnvironmentalDataAuthority,
    EnvironmentalDataProvider,
    EnvironmentalEvent,
    EnvironmentalInformation,
    EnvironmentalLocationType,
    EnvironmentalPhenomenon,
    EnvironmentalStringMeasurement,
    Equipment,
    EquipmentContainer,
    EquipmentFault,
    Estimate,
    ExtensionItem,
    ExtensionsList,
    ExternalNetworkInjection,
    FACTSDevice,
    FScale,
    Facility,
    FacilityKind,
    FailureEvent,
    FailureIsolationMethodKind,
    Fault,
    FaultCauseType,
    FaultImpedance,
    Feeder,
    FieldDispatchHistory,
    FieldDispatchStep,
    FinancialInfo,
    Fire,
    FloatQuantity,
    Flood,
    FlowDirectionKind,
    Forecast,
    FossilFuel,
    FrequencyConverter,
    FuelAllocationSchedule,
    FuelType,
    Fuse,
    GenUnitOpCostCurve,
    GenUnitOpSchedule,
    GeneratingUnit,
    GeographicalRegion,
    GeosphericAnalog,
    GeosphericAnalogKind,
    GeosphericPhenomenon,
    GrossToNetActivePowerCurve,
    Ground,
    GroundDisconnector,
    GroundingImpedance,
    Hazard,
    HealthScore,
    HeatInputCurve,
    HeatRateCurve,
    House,
    HouseCooling,
    HouseHeating,
    Hurricane,
    HydroEnergyConversionKind,
    HydroGeneratingEfficiencyCurve,
    HydroGeneratingUnit,
    HydroPlantStorageKind,
    HydroPowerPlant,
    HydroPump,
    HydroPumpOpSchedule,
    HydrosphericAnalog,
    HydrosphericAnalogKind,
    HydrosphericPhenomenon,
    IEEE1547AbnormalPerfomanceCategory,
    IEEE1547ControlSettings,
    IEEE1547Info,
    IEEE1547IslandingCategory,
    IEEE1547NormalPerformanceCategory,
    IEEE1547Setting,
    IEEE1547TripSettings,
    IOPoint,
    IdentifiedObject,
    InUseDate,
    IncidentHazard,
    IncrementalHeatRateCurve,
    InflowForecast,
    InspectionDataSet,
    IntegerQuantity,
    IntensityCodeKind,
    InterrupterUnit,
    InterrupterUnitInfo,
    InterruptingMediumKind,
    IntervalBlock,
    IntervalReading,
    IrregularIntervalSchedule,
    IrregularTimePoint,
    Issuer,
    Joint,
    Jumper,
    Junction,
    LabTestDataSet,
    Landslide,
    LevelVsVolumeCurve,
    LifecycleDate,
    LightningStrike,
    Limit,
    LimitSet,
    Line,
    LineFault,
    LinearShuntCompensator,
    LinearShuntCompensatorPhase,
    LoadArea,
    LoadBreakSwitch,
    LoadGroup,
    LoadResponseCharacteristic,
    Location,
    LocationKind,
    MacroPeriodKind,
    MagneticStorm,
    Maintainer,
    MaintenanceDataSet,
    Manufacturer,
    Measurement,
    MeasurementKind,
    MeasurementValue,
    MeasurementValueQuality,
    MeasurementValueSource,
    MeasuringPeriodKind,
    Medium,
    MediumKind,
    Meter,
    MeterMultiplier,
    MeterMultiplierKind,
    MeterReading,
    MeterWorkTask,
    MetrologyRequirement,
    MonthDayInterval,
    MutualCoupling,
    Name,
    NameType,
    NameTypeAuthority,
    NamedPhenomenon,
    NoLoadTest,
    NonConformLoad,
    NonConformLoadGroup,
    NonConformLoadSchedule,
    NonlinearShuntCompensator,
    NonlinearShuntCompensatorPhase,
    NonlinearShuntCompensatorPhasePoint,
    NonlinearShuntCompensatorPoint,
    NotificationTriggerKind,
    NuclearGeneratingUnit,
    Observation,
    OilSampleLocation,
    OilSpecimen,
    OilTemperatureSource,
    OpenCircuitTest,
    OperatingMechanism,
    OperatingMechanismInfo,
    OperatingMechanismKind,
    OperatingParticipant,
    OperatingShare,
    OperationPersonRole,
    OperationalLimit,
    OperationalLimitDirectionKind,
    OperationalLimitSet,
    OperationalLimitType,
    Operator,
    Organisation,
    OrganisationRole,
    OverfrequencyTripCurve,
    OverfrequencyTripCurveData,
    OverheadWireInfo,
    OvervoltageTripCurve,
    OvervoltageTripCurveData,
    Ownership,
    PNNLTroubleCallKind,
    PSRType,
    PanDemandResponse,
    PanDisplay,
    PanPricing,
    PanPricingDetail,
    ParallelLineSegment,
    ParentOrganization,
    PendingCalculation,
    PenstockLossCurve,
    PerLengthImpedance,
    PerLengthLineParameter,
    PerLengthPhaseImpedance,
    PerLengthSequenceImpedance,
    Person,
    PersonRole,
    PetersenCoil,
    PetersenCoilModeKind,
    PhaseCode,
    PhaseConnectedFaultKind,
    PhaseImpedanceData,
    PhaseShuntConnectionKind,
    PhaseTapChanger,
    PhaseTapChangerAsymmetrical,
    PhaseTapChangerLinear,
    PhaseTapChangerNonLinear,
    PhaseTapChangerSymmetrical,
    PhaseTapChangerTable,
    PhaseTapChangerTablePoint,
    PhaseTapChangerTabular,
    PhenomenonTypeification,
    PhotovoltaicUnit,
    Plant,
    PositionPoint,
    PowerCutZone,
    PowerElectronicsConnection,
    PowerElectronicsConnectionPhase,
    PowerElectronicsUnit,
    PowerElectronicsWindUnit,
    PowerSystemResource,
    PowerTransformer,
    PowerTransformerEnd,
    PowerTransformerInfo,
    PricingStructure,
    Priority,
    Procedure,
    ProcedureDataSet,
    ProcedureKind,
    ProductAssetModel,
    ProtectedSwitch,
    Quality61850,
    RaiseLowerCommand,
    RandomisationKind,
    RatioTapChanger,
    RatioTapChangerTable,
    RatioTapChangerTablePoint,
    RationalNumber,
    ReactiveCapabilityCurve,
    Reading,
    ReadingInterharmonic,
    ReadingQuality,
    ReadingQualityType,
    ReadingReasonKind,
    ReadingType,
    Recloser,
    Register,
    RegularIntervalSchedule,
    RegularTimePoint,
    RegulatingCondEq,
    RegulatingControl,
    RegulatingControlModeKind,
    RegulationSchedule,
    RelativeDisplacement,
    RelativeDisplacementKind,
    ReportingCapability,
    ReportingGroup,
    ReportingMethodKind,
    ReportingSuperGroup,
    Reservoir,
    RevenueKind,
    RightOfWay,
    RiskScore,
    RiskScoreKind,
    RotatingMachine,
    SVCControlMode,
    SampleContainerType,
    ScaleKind,
    ScheduledEvent,
    ScheduledEventData,
    Seal,
    SealConditionKind,
    SealKind,
    Season,
    SeasonDayTypeSchedule,
    Sectionaliser,
    SeriesCompensator,
    ServiceCategory,
    ServiceKind,
    ServiceLocation,
    ServiceMultiplier,
    ServiceMultiplierKind,
    SetPoint,
    ShortCircuitTest,
    ShuntCompensator,
    ShuntCompensatorInfo,
    ShuntCompensatorPhase,
    ShutdownCurve,
    SimpleEndDeviceFunction,
    SinglePhaseKind,
    SmartInverterMode,
    SolarGeneratingUnit,
    SpaceAnalog,
    SpaceAnalogKind,
    SpacePhenomenon,
    Specimen,
    StartIgnFuelCurve,
    StartMainFuelCurve,
    StartRampCurve,
    StartupModel,
    StateVariable,
    StaticVarCompensator,
    StationSupply,
    Status,
    SteamSendoutSchedule,
    StreetAddress,
    StreetDetail,
    Streetlight,
    StringMeasurement,
    StringMeasurementValue,
    StringQuantity,
    Structure,
    StructureSupport,
    SubGeographicalRegion,
    SubLoadArea,
    Substation,
    SvEstVoltage,
    SvInjection,
    SvPowerFlow,
    SvShuntCompensatorSections,
    SvStatus,
    SvSwitch,
    SvTapStep,
    SvVoltage,
    Switch,
    SwitchInfo,
    SwitchOperationSummary,
    SwitchPhase,
    SwitchSchedule,
    SynchronousMachine,
    SynchronousMachineKind,
    SynchronousMachineOperatingMode,
    TailbayLossCurve,
    TapChanger,
    TapChangerControl,
    TapChangerInfo,
    TapChangerTablePoint,
    TapSchedule,
    TapeShieldCableInfo,
    TargetLevelSchedule,
    Tariff,
    TelephoneNumber,
    Terminal,
    TestDataSet,
    TestKind,
    TestMethod,
    TestReason,
    TestStandard,
    TestVariantKind,
    ThermalGeneratingUnit,
    ThermostatControlMode,
    ThermostatController,
    TimeInterval,
    TimeIntervalKind,
    TimePoint,
    TimeSchedule,
    TopologicalIsland,
    TopologicalNode,
    Tornado,
    TownDetail,
    TransformerApplicationKind,
    TransformerCoreAdmittance,
    TransformerEnd,
    TransformerEndInfo,
    TransformerFailureReasonKind,
    TransformerMeshImpedance,
    TransformerStarImpedance,
    TransformerTank,
    TransformerTankEnd,
    TransformerTankInfo,
    TransformerTest,
    TransmissionModeKind,
    TropicalCycloneAustralia,
    TroubleReportingKind,
    TroubleTicket,
    Tsunami,
    UnderfrequencyTripCurve,
    UnderfrequencyTripCurveData,
    UndervoltageTripCurve,
    UndervoltageTripCurveData,
    UnitMultiplier,
    UnitSymbol,
    UsagePoint,
    UsagePointConnectedKind,
    UsagePointGroup,
    UsagePointLocation,
    UserAttribute,
    Validity,
    ValueAliasSet,
    ValueToAlias,
    Version,
    VolcanicAshCloud,
    VoltVarCurve,
    VoltVarCurveData,
    VoltWattCurve,
    VoltWattCurveData,
    VoltageControlZone,
    VoltageLevel,
    VoltageLimit,
    WattVarCurve,
    WattVarCurveData,
    WeatherCodeKind,
    Whirlpool,
    WindGenUnitKind,
    WindGeneratingUnit,
    WindingConnection,
    WireAssemblyInfo,
    WireInfo,
    WireInsulationKind,
    WireMaterialKind,
    WirePhaseInfo,
    WirePosition,
    WireSpacingInfo,
    WireUsageKind,
)

__all__ = [
    "ACDCTerminal",
    "ACLineSegment",
    "ACLineSegmentPhase",
    "AcceptanceTest",
    "AccountNotification",
    "AccumulationKind",
    "Accumulator",
    "AccumulatorLimit",
    "AccumulatorLimitSet",
    "AccumulatorReset",
    "AccumulatorValue",
    "ActivePowerLimit",
    "ActivityRecord",
    "AggregateKind",
    "AggregateScore",
    "Agreement",
    "AirCompressor",
    "AlertTypeList",
    "AmiBillingReadyKind",
    "Analog",
    "AnalogControl",
    "AnalogLimit",
    "AnalogLimitSet",
    "AnalogValue",
    "Analytic",
    "AnalyticKind",
    "AnalyticScore",
    "ApparentPowerLimit",
    "Appointment",
    "Approver",
    "Asset",
    "AssetContainer",
    "AssetDeployment",
    "AssetFailureTypeification",
    "AssetFailureMode",
    "AssetFunction",
    "AssetGroup",
    "AssetGroupKind",
    "AssetHazardKind",
    "AssetHealthEvent",
    "AssetInfo",
    "AssetKind",
    "AssetLocationHazard",
    "AssetModelUsageKind",
    "AssetOrganisationRole",
    "AssetOwner",
    "AssetTestLab",
    "AssetTestSampleTaker",
    "AssetUser",
    "AsynchronousMachine",
    "AsynchronousMachineKind",
    "AtmosphericAnalog",
    "AtmosphericAnalogKind",
    "AtmosphericPhenomenon",
    "Author",
    "BaseFrequency",
    "BasePower",
    "BaseReading",
    "BaseVoltage",
    "BasicIntervalSchedule",
    "BatteryState",
    "BatteryUnit",
    "Bay",
    "BranchGroup",
    "BranchGroupTerminal",
    "Breaker",
    "BreakerApplicationKind",
    "BreakerConfiguration",
    "BreakerFailureReasonKind",
    "BusNameMarker",
    "BusbarConfiguration",
    "BusbarSection",
    "BusbarSectionInfo",
    "Bushing",
    "BushingInfo",
    "BushingInsulationKind",
    "CAESPlant",
    "CIM_GridAPPS_D_RC4_2021",
    "Cabinet",
    "CableConstructionKind",
    "CableInfo",
    "CableOuterJacketKind",
    "CableShieldMaterialKind",
    "CatalogAssetType",
    "Channel",
    "Clamp",
    "TypeificationCondition",
    "CloudCondition",
    "CloudKind",
    "CogenerationPlant",
    "ComDirectionKind",
    "ComFunction",
    "ComMedia",
    "ComModule",
    "ComTechnologyKind",
    "CombinedCyclePlant",
    "Command",
    "CommodityKind",
    "CompositeSwitch",
    "ConcentricNeutralCableInfo",
    "ConductingEquipment",
    "Conductor",
    "ConfigurationEvent",
    "ConformLoad",
    "ConformLoadGroup",
    "ConformLoadSchedule",
    "ConnectivityNode",
    "ConnectivityNodeContainer",
    "Connector",
    "Control",
    "ControlledAppliance",
    "CoordinateSystem",
    "CorporateStandardKind",
    "CoverageCodeKind",
    "Crew",
    "CrewMember",
    "CrewStatusKind",
    "CrewType",
    "Currency",
    "CurrentLimit",
    "Curve",
    "CurveData",
    "CurveStyle",
    "Customer",
    "CustomerAccount",
    "CustomerAgreement",
    "CustomerKind",
    "CustomerNotification",
    "Cut",
    "Cyclone",
    "DERCurveData",
    "DERFunction",
    "DERGroupDispatch",
    "DERGroupForecast",
    "DERMonitorableParameter",
    "DERParameterKind",
    "DERUnitSymbol",
    "DateInterval",
    "DateTimeInterval",
    "DayType",
    "DecimalQuantity",
    "DemandResponseProgram",
    "DeploymentDate",
    "DeploymentStateKind",
    "DiagnosisDataSet",
    "Disconnector",
    "Discrete",
    "DiscreteCommand",
    "DiscreteValue",
    "DispatchSchedule",
    "DispatchablePowerCapability",
    "Document",
    "DocumentPersonRole",
    "DuctBank",
    "EarthFaultCompensator",
    "Earthquake",
    "Editor",
    "ElectronicAddress",
    "EmissionAccount",
    "EmissionCurve",
    "EmissionType",
    "EmissionValueSource",
    "EndDevice",
    "EndDeviceAction",
    "EndDeviceCapability",
    "EndDeviceControl",
    "EndDeviceControlType",
    "EndDeviceEvent",
    "EndDeviceEventDetail",
    "EndDeviceEventType",
    "EndDeviceFunction",
    "EndDeviceFunctionKind",
    "EndDeviceGroup",
    "EndDeviceInfo",
    "EndDeviceTiming",
    "EnergyArea",
    "EnergyConnection",
    "EnergyConsumer",
    "EnergyConsumerPhase",
    "EnergySource",
    "EnergySourcePhase",
    "EnvironmentalAlert",
    "EnvironmentalAnalog",
    "EnvironmentalCodedValue",
    "EnvironmentalDataAuthority",
    "EnvironmentalDataProvider",
    "EnvironmentalEvent",
    "EnvironmentalInformation",
    "EnvironmentalLocationType",
    "EnvironmentalPhenomenon",
    "EnvironmentalStringMeasurement",
    "Equipment",
    "EquipmentContainer",
    "EquipmentFault",
    "Estimate",
    "ExtensionItem",
    "ExtensionsList",
    "ExternalNetworkInjection",
    "FACTSDevice",
    "FScale",
    "Facility",
    "FacilityKind",
    "FailureEvent",
    "FailureIsolationMethodKind",
    "Fault",
    "FaultCauseType",
    "FaultImpedance",
    "Feeder",
    "FieldDispatchHistory",
    "FieldDispatchStep",
    "FinancialInfo",
    "Fire",
    "FloatQuantity",
    "Flood",
    "FlowDirectionKind",
    "Forecast",
    "FossilFuel",
    "FrequencyConverter",
    "FuelAllocationSchedule",
    "FuelType",
    "Fuse",
    "GenUnitOpCostCurve",
    "GenUnitOpSchedule",
    "GeneratingUnit",
    "GeographicalRegion",
    "GeosphericAnalog",
    "GeosphericAnalogKind",
    "GeosphericPhenomenon",
    "GrossToNetActivePowerCurve",
    "Ground",
    "GroundDisconnector",
    "GroundingImpedance",
    "Hazard",
    "HealthScore",
    "HeatInputCurve",
    "HeatRateCurve",
    "House",
    "HouseCooling",
    "HouseHeating",
    "Hurricane",
    "HydroEnergyConversionKind",
    "HydroGeneratingEfficiencyCurve",
    "HydroGeneratingUnit",
    "HydroPlantStorageKind",
    "HydroPowerPlant",
    "HydroPump",
    "HydroPumpOpSchedule",
    "HydrosphericAnalog",
    "HydrosphericAnalogKind",
    "HydrosphericPhenomenon",
    "IEEE1547AbnormalPerfomanceCategory",
    "IEEE1547ControlSettings",
    "IEEE1547Info",
    "IEEE1547IslandingCategory",
    "IEEE1547NormalPerformanceCategory",
    "IEEE1547Setting",
    "IEEE1547TripSettings",
    "IOPoint",
    "IdentifiedObject",
    "InUseDate",
    "IncidentHazard",
    "IncrementalHeatRateCurve",
    "InflowForecast",
    "InspectionDataSet",
    "IntegerQuantity",
    "IntensityCodeKind",
    "InterrupterUnit",
    "InterrupterUnitInfo",
    "InterruptingMediumKind",
    "IntervalBlock",
    "IntervalReading",
    "IrregularIntervalSchedule",
    "IrregularTimePoint",
    "Issuer",
    "Joint",
    "Jumper",
    "Junction",
    "LabTestDataSet",
    "LaborelecStandard",
    "LaborelecStandardEditionKind",
    "LaborelecStandardKind",
    "Landslide",
    "LevelVsVolumeCurve",
    "LifecycleDate",
    "LightningStrike",
    "Limit",
    "LimitSet",
    "Line",
    "LineFault",
    "LinearShuntCompensator",
    "LinearShuntCompensatorPhase",
    "LoadArea",
    "LoadBreakSwitch",
    "LoadGroup",
    "LoadResponseCharacteristic",
    "Location",
    "LocationKind",
    "MacroPeriodKind",
    "MagneticStorm",
    "Maintainer",
    "MaintenanceDataSet",
    "Manufacturer",
    "Measurement",
    "MeasurementKind",
    "MeasurementValue",
    "MeasurementValueQuality",
    "MeasurementValueSource",
    "MeasuringPeriodKind",
    "Medium",
    "MediumKind",
    "Meter",
    "MeterMultiplier",
    "MeterMultiplierKind",
    "MeterReading",
    "MeterWorkTask",
    "MetrologyRequirement",
    "MonthDayInterval",
    "MutualCoupling",
    "Name",
    "NameType",
    "NameTypeAuthority",
    "NamedPhenomenon",
    "NoLoadTest",
    "NonConformLoad",
    "NonConformLoadGroup",
    "NonConformLoadSchedule",
    "NonlinearShuntCompensator",
    "NonlinearShuntCompensatorPhase",
    "NonlinearShuntCompensatorPhasePoint",
    "NonlinearShuntCompensatorPoint",
    "NotificationTriggerKind",
    "NuclearGeneratingUnit",
    "Observation",
    "OilSampleLocation",
    "OilSpecimen",
    "OilTemperatureSource",
    "OpenCircuitTest",
    "OperatingMechanism",
    "OperatingMechanismInfo",
    "OperatingMechanismKind",
    "OperatingParticipant",
    "OperatingShare",
    "OperationPersonRole",
    "OperationalLimit",
    "OperationalLimitDirectionKind",
    "OperationalLimitSet",
    "OperationalLimitType",
    "Operator",
    "Organisation",
    "OrganisationRole",
    "OverfrequencyTripCurve",
    "OverfrequencyTripCurveData",
    "OverheadWireInfo",
    "OvervoltageTripCurve",
    "OvervoltageTripCurveData",
    "Ownership",
    "PNNLTroubleCallKind",
    "PSRType",
    "PanDemandResponse",
    "PanDisplay",
    "PanPricing",
    "PanPricingDetail",
    "ParallelLineSegment",
    "ParentOrganization",
    "PendingCalculation",
    "PenstockLossCurve",
    "PerLengthImpedance",
    "PerLengthLineParameter",
    "PerLengthPhaseImpedance",
    "PerLengthSequenceImpedance",
    "Person",
    "PersonRole",
    "PetersenCoil",
    "PetersenCoilModeKind",
    "PhaseCode",
    "PhaseConnectedFaultKind",
    "PhaseImpedanceData",
    "PhaseShuntConnectionKind",
    "PhaseTapChanger",
    "PhaseTapChangerAsymmetrical",
    "PhaseTapChangerLinear",
    "PhaseTapChangerNonLinear",
    "PhaseTapChangerSymmetrical",
    "PhaseTapChangerTable",
    "PhaseTapChangerTablePoint",
    "PhaseTapChangerTabular",
    "PhenomenonTypeification",
    "PhotovoltaicUnit",
    "Plant",
    "PositionPoint",
    "PowerCutZone",
    "PowerElectronicsConnection",
    "PowerElectronicsConnectionPhase",
    "PowerElectronicsUnit",
    "PowerElectronicsWindUnit",
    "PowerSystemResource",
    "PowerTransformer",
    "PowerTransformerEnd",
    "PowerTransformerInfo",
    "PricingStructure",
    "Priority",
    "Procedure",
    "ProcedureDataSet",
    "ProcedureKind",
    "ProductAssetModel",
    "ProtectedSwitch",
    "Quality61850",
    "RaiseLowerCommand",
    "RandomisationKind",
    "RatioTapChanger",
    "RatioTapChangerTable",
    "RatioTapChangerTablePoint",
    "RationalNumber",
    "ReactiveCapabilityCurve",
    "Reading",
    "ReadingInterharmonic",
    "ReadingQuality",
    "ReadingQualityType",
    "ReadingReasonKind",
    "ReadingType",
    "Recloser",
    "Register",
    "RegularIntervalSchedule",
    "RegularTimePoint",
    "RegulatingCondEq",
    "RegulatingControl",
    "RegulatingControlModeKind",
    "RegulationSchedule",
    "RelativeDisplacement",
    "RelativeDisplacementKind",
    "ReportingCapability",
    "ReportingGroup",
    "ReportingMethodKind",
    "ReportingSuperGroup",
    "Reservoir",
    "RevenueKind",
    "RightOfWay",
    "RiskScore",
    "RiskScoreKind",
    "RotatingMachine",
    "SVCControlMode",
    "SampleContainerType",
    "ScaleKind",
    "ScheduledEvent",
    "ScheduledEventData",
    "Seal",
    "SealConditionKind",
    "SealKind",
    "Season",
    "SeasonDayTypeSchedule",
    "Sectionaliser",
    "SeriesCompensator",
    "ServiceCategory",
    "ServiceKind",
    "ServiceLocation",
    "ServiceMultiplier",
    "ServiceMultiplierKind",
    "SetPoint",
    "ShortCircuitTest",
    "ShuntCompensator",
    "ShuntCompensatorInfo",
    "ShuntCompensatorPhase",
    "ShutdownCurve",
    "SimpleEndDeviceFunction",
    "SinglePhaseKind",
    "SmartInverterMode",
    "SolarGeneratingUnit",
    "SpaceAnalog",
    "SpaceAnalogKind",
    "SpacePhenomenon",
    "Specimen",
    "StartIgnFuelCurve",
    "StartMainFuelCurve",
    "StartRampCurve",
    "StartupModel",
    "StateVariable",
    "StaticVarCompensator",
    "StationSupply",
    "Status",
    "SteamSendoutSchedule",
    "StreetAddress",
    "StreetDetail",
    "Streetlight",
    "StringMeasurement",
    "StringMeasurementValue",
    "StringQuantity",
    "Structure",
    "StructureSupport",
    "SubGeographicalRegion",
    "SubLoadArea",
    "Substation",
    "SvEstVoltage",
    "SvInjection",
    "SvPowerFlow",
    "SvShuntCompensatorSections",
    "SvStatus",
    "SvSwitch",
    "SvTapStep",
    "SvVoltage",
    "Switch",
    "SwitchInfo",
    "SwitchOperationSummary",
    "SwitchPhase",
    "SwitchSchedule",
    "SynchronousMachine",
    "SynchronousMachineKind",
    "SynchronousMachineOperatingMode",
    "TailbayLossCurve",
    "TapChanger",
    "TapChangerControl",
    "TapChangerInfo",
    "TapChangerTablePoint",
    "TapSchedule",
    "TapeShieldCableInfo",
    "TargetLevelSchedule",
    "Tariff",
    "TelephoneNumber",
    "Terminal",
    "TestDataSet",
    "TestKind",
    "TestMethod",
    "TestReason",
    "TestStandard",
    "TestVariantKind",
    "ThermalGeneratingUnit",
    "ThermostatControlMode",
    "ThermostatController",
    "TimeInterval",
    "TimeIntervalKind",
    "TimePoint",
    "TimeSchedule",
    "TopologicalIsland",
    "TopologicalNode",
    "Tornado",
    "TownDetail",
    "TransformerApplicationKind",
    "TransformerCoreAdmittance",
    "TransformerEnd",
    "TransformerEndInfo",
    "TransformerFailureReasonKind",
    "TransformerMeshImpedance",
    "TransformerStarImpedance",
    "TransformerTank",
    "TransformerTankEnd",
    "TransformerTankInfo",
    "TransformerTest",
    "TransmissionModeKind",
    "TropicalCycloneAustralia",
    "TroubleReportingKind",
    "TroubleTicket",
    "Tsunami",
    "UnderfrequencyTripCurve",
    "UnderfrequencyTripCurveData",
    "UndervoltageTripCurve",
    "UndervoltageTripCurveData",
    "UnitMultiplier",
    "UnitSymbol",
    "UsagePoint",
    "UsagePointConnectedKind",
    "UsagePointGroup",
    "UsagePointLocation",
    "UserAttribute",
    "Validity",
    "ValueAliasSet",
    "ValueToAlias",
    "Version",
    "VolcanicAshCloud",
    "VoltVarCurve",
    "VoltVarCurveData",
    "VoltWattCurve",
    "VoltWattCurveData",
    "VoltageControlZone",
    "VoltageLevel",
    "VoltageLimit",
    "WattVarCurve",
    "WattVarCurveData",
    "WeatherCodeKind",
    "Whirlpool",
    "WindGenUnitKind",
    "WindGeneratingUnit",
    "WindingConnection",
    "WireAssemblyInfo",
    "WireInfo",
    "WireInsulationKind",
    "WireMaterialKind",
    "WirePhaseInfo",
    "WirePosition",
    "WireSpacingInfo",
    "WireUsageKind",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cimlab',
 'cimlab.data_profile',
 'cimlab.data_profile.cimext_2022',
 'cimlab.data_profile.rc4_2021',
 'cimlab.loaders',
 'cimlab.loaders.blazegraph',
 'cimlab.loaders.gridappsd',
 'cimlab.loaders.sparql.rc4_2021',
 'cimlab.models']

package_data = \
{'': ['*']}

install_requires = \
['SPARQLWrapper>=2.0.0,<3.0.0', 'xsdata>=22.5,<23.0']

extras_require = \
{'gridappsd': ['gridappsd-python>=2.2.2,<3.0.0']}

setup_kwargs = {
    'name': 'gridappsd-cim-lab',
    'version': '0.11.230209',
    'description': 'CIM models used within gridappsd.',
    'long_description': 'None',
    'author': 'C. Allwardt',
    'author_email': '3979063+craig8@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.9,<4.0',
}


setup(**setup_kwargs)



from cimlab.data_profile.cimext_2022.gridappsd_cim_profile import (
    ACDCTerminal,
    ACLineSegment,
    ACLineSegmentPhase,
    AbnormalOPcatKind,
    ActivityRecord,
    AmiBillingReadyKind,
    Asset,
    AssetContainer,
    AssetFunction,
    AssetInfo,
    BaseVoltage,
    BasicIntervalSchedule,
    Breaker,
    BusbarSection,
    BusbarSectionInfo,
    CableConstructionKind,
    CableInfo,
    CableOuterJacketKind,
    CableShieldMaterialKind,
    ConcentricNeutralCableInfo,
    ConductingEquipment,
    Conductor,
    ConfigurationEvent,
    ConnectivityNode,
    ConnectivityNodeContainer,
    Connector,
    ConstantPowerFactorSettingKind,
    ConstantPowerFactorSettings,
    ConstantReactivePowerSettings,
    Control,
    ConverterControlMode,
    CoordinateSystem,
    Curve,
    CurveData,
    CurveStyle,
    Customer,
    DERCurveData,
    DERDynamics,
    DERFunction,
    DERGroupDispatch,
    DERGroupForecast,
    DERIEEEType1,
    DERMonitorableParameter,
    DERNameplateData,
    DERNameplateDataApplied,
    DERParameterKind,
    DERUnitSymbol,
    DateTimeInterval,
    DayType,
    DemandResponseProgram,
    DispatchSchedule,
    DispatchablePowerCapability,
    DynamicsFunctionBlock,
    EndDevice,
    EndDeviceAction,
    EndDeviceCapability,
    EndDeviceControl,
    EndDeviceControlType,
    EndDeviceEvent,
    EndDeviceEventDetail,
    EndDeviceEventType,
    EndDeviceFunction,
    EndDeviceGroup,
    EndDeviceInfo,
    EndDeviceTiming,
    EnergyConnection,
    EnergyConsumer,
    EnergyConsumerPhase,
    EnergySource,
    EnergySourcePhase,
    Equipment,
    EquipmentContainer,
    EquipmentFault,
    Estimate,
    Fault,
    FaultCauseType,
    FaultImpedance,
    Feeder,
    FloatQuantity,
    FlowDirectionKind,
    FrequencyDroopSettings,
    FrequencyTripSettings,
    Fuse,
    GAPPS_CIM_export,
    GeographicalRegion,
    House,
    HouseCooling,
    HouseHeating,
    IEEE1547AbnormalPerfomanceCategory,
    IEEE1547ControlSettings,
    IEEE1547Info,
    IEEE1547IslandingCategory,
    IEEE1547NormalPerformanceCategory,
    IEEE1547Setting,
    IEEE1547TripSettings,
    IOPoint,
    IdentifiedObject,
    LinearShuntCompensator,
    LinearShuntCompensatorPhase,
    LoadBreakSwitch,
    LoadResponseCharacteristic,
    Location,
    Measurement,
    Meter,
    MeterReading,
    MomentaryCessationSettings,
    Name,
    NoLoadTest,
    NormalOPcatKind,
    OpenCircuitTest,
    OperationalLimit,
    OperationalLimitDirectionKind,
    OperationalLimitSet,
    OperationalLimitType,
    OrderedPhaseCodeKind,
    OverfrequencyTripCurve,
    OverheadWireInfo,
    OvervoltageTripCurve,
    PerLengthImpedance,
    PerLengthLineParameter,
    PerLengthSequenceImpedance,
    PhaseCode,
    PhaseConnectedFaultKind,
    PhaseShuntConnectionKind,
    PhaseTapChanger,
    PowerCutZone,
    PowerElectronicsConnection,
    PowerElectronicsConnectionPhase,
    PowerElectronicsUnit,
    PowerLimitSettings,
    PowerSystemResource,
    PowerTransformer,
    PowerTransformerEnd,
    PowerTransformerInfo,
    Priority,
    ProtectedSwitch,
    RandomisationKind,
    RatioTapChanger,
    ReactiveCapabilityCurve,
    Recloser,
    RegularIntervalSchedule,
    RegularTimePoint,
    RegulatingCondEq,
    RegulatingControl,
    RegulatingControlModeKind,
    RegulationSchedule,
    RemoteInputSignal,
    RemoteSignalKind,
    RotatingMachine,
    ScheduledEvent,
    Season,
    SeasonDayTypeSchedule,
    Sectionaliser,
    ServiceSettings,
    ShortCircuitTest,
    ShuntCompensator,
    ShuntCompensatorInfo,
    ShuntCompensatorPhase,
    SinglePhaseKind,
    StateVariable,
    Status,
    StringQuantity,
    SubGeographicalRegion,
    Substation,
    SvEstVoltage,
    SvInjection,
    SvPowerFlow,
    SvShuntCompensatorSections,
    SvStatus,
    SvTapStep,
    SvVoltage,
    Switch,
    SwitchInfo,
    SynchronousMachine,
    TapChanger,
    TapChangerControl,
    TapChangerInfo,
    TapSchedule,
    TapeShieldCableInfo,
    Terminal,
    ThermostatControlMode,
    ThermostatController,
    TimeIntervalKind,
    TopologicalIsland,
    TopologicalNode,
    TransformerCoreAdmittance,
    TransformerEnd,
    TransformerEndInfo,
    TransformerMeshImpedance,
    TransformerTank,
    TransformerTankEnd,
    TransformerTankInfo,
    TransformerTest,
    UnderfrequencyTripCurve,
    UndervoltageTripCurve,
    UnitMultiplier,
    UnitSymbol,
    UsagePoint,
    UsagePointConnectedKind,
    UsagePointGroup,
    Version,
    VoltVarCurve,
    VoltVarSettings,
    VoltWattCurve,
    VoltWattSettings,
    VoltageControlZone,
    VoltageTripSettings,
    WattVarCurve,
    WattVarSettings,
    WindingConnection,
    WireAssemblyInfo,
    WireInfo,
    WireInsulationKind,
    WireMaterialKind,
    WirePhaseInfo,
    WirePosition,
    WireSpacingInfo,
)

__all__ = [
    "ACDCTerminal",
    "ACLineSegment",
    "ACLineSegmentPhase",
    "AbnormalOPcatKind",
    "ActivityRecord",
    "AmiBillingReadyKind",
    "Asset",
    "AssetContainer",
    "AssetFunction",
    "AssetInfo",
    "BaseVoltage",
    "BasicIntervalSchedule",
    "Breaker",
    "BusbarSection",
    "BusbarSectionInfo",
    "CableConstructionKind",
    "CableInfo",
    "CableOuterJacketKind",
    "CableShieldMaterialKind",
    "ConcentricNeutralCableInfo",
    "ConductingEquipment",
    "Conductor",
    "ConfigurationEvent",
    "ConnectivityNode",
    "ConnectivityNodeContainer",
    "Connector",
    "ConstantPowerFactorSettingKind",
    "ConstantPowerFactorSettings",
    "ConstantReactivePowerSettings",
    "Control",
    "ConverterControlMode",
    "CoordinateSystem",
    "Curve",
    "CurveData",
    "CurveStyle",
    "Customer",
    "DERCurveData",
    "DERDynamics",
    "DERFunction",
    "DERGroupDispatch",
    "DERGroupForecast",
    "DERIEEEType1",
    "DERMonitorableParameter",
    "DERNameplateData",
    "DERNameplateDataApplied",
    "DERParameterKind",
    "DERUnitSymbol",
    "DateTimeInterval",
    "DayType",
    "DemandResponseProgram",
    "DispatchSchedule",
    "DispatchablePowerCapability",
    "DynamicsFunctionBlock",
    "EndDevice",
    "EndDeviceAction",
    "EndDeviceCapability",
    "EndDeviceControl",
    "EndDeviceControlType",
    "EndDeviceEvent",
    "EndDeviceEventDetail",
    "EndDeviceEventType",
    "EndDeviceFunction",
    "EndDeviceGroup",
    "EndDeviceInfo",
    "EndDeviceTiming",
    "EnergyConnection",
    "EnergyConsumer",
    "EnergyConsumerPhase",
    "EnergySource",
    "EnergySourcePhase",
    "Equipment",
    "EquipmentContainer",
    "EquipmentFault",
    "Estimate",
    "Fault",
    "FaultCauseType",
    "FaultImpedance",
    "Feeder",
    "FloatQuantity",
    "FlowDirectionKind",
    "FrequencyDroopSettings",
    "FrequencyTripSettings",
    "Fuse",
    "GAPPS_CIM_export",
    "GeographicalRegion",
    "House",
    "HouseCooling",
    "HouseHeating",
    "IEEE1547AbnormalPerfomanceCategory",
    "IEEE1547ControlSettings",
    "IEEE1547Info",
    "IEEE1547IslandingCategory",
    "IEEE1547NormalPerformanceCategory",
    "IEEE1547Setting",
    "IEEE1547TripSettings",
    "IOPoint",
    "IdentifiedObject",
    "LinearShuntCompensator",
    "LinearShuntCompensatorPhase",
    "LoadBreakSwitch",
    "LoadResponseCharacteristic",
    "Location",
    "Measurement",
    "Meter",
    "MeterReading",
    "MomentaryCessationSettings",
    "Name",
    "NoLoadTest",
    "NormalOPcatKind",
    "OpenCircuitTest",
    "OperationalLimit",
    "OperationalLimitDirectionKind",
    "OperationalLimitSet",
    "OperationalLimitType",
    "OrderedPhaseCodeKind",
    "OverfrequencyTripCurve",
    "OverheadWireInfo",
    "OvervoltageTripCurve",
    "PerLengthImpedance",
    "PerLengthLineParameter",
    "PerLengthSequenceImpedance",
    "PhaseCode",
    "PhaseConnectedFaultKind",
    "PhaseShuntConnectionKind",
    "PhaseTapChanger",
    "PowerCutZone",
    "PowerElectronicsConnection",
    "PowerElectronicsConnectionPhase",
    "PowerElectronicsUnit",
    "PowerLimitSettings",
    "PowerSystemResource",
    "PowerTransformer",
    "PowerTransformerEnd",
    "PowerTransformerInfo",
    "Priority",
    "ProtectedSwitch",
    "RandomisationKind",
    "RatioTapChanger",
    "ReactiveCapabilityCurve",
    "Recloser",
    "RegularIntervalSchedule",
    "RegularTimePoint",
    "RegulatingCondEq",
    "RegulatingControl",
    "RegulatingControlModeKind",
    "RegulationSchedule",
    "RemoteInputSignal",
    "RemoteSignalKind",
    "RotatingMachine",
    "ScheduledEvent",
    "Season",
    "SeasonDayTypeSchedule",
    "Sectionaliser",
    "ServiceSettings",
    "ShortCircuitTest",
    "ShuntCompensator",
    "ShuntCompensatorInfo",
    "ShuntCompensatorPhase",
    "SinglePhaseKind",
    "StateVariable",
    "Status",
    "StringQuantity",
    "SubGeographicalRegion",
    "Substation",
    "SvEstVoltage",
    "SvInjection",
    "SvPowerFlow",
    "SvShuntCompensatorSections",
    "SvStatus",
    "SvTapStep",
    "SvVoltage",
    "Switch",
    "SwitchInfo",
    "SynchronousMachine",
    "TapChanger",
    "TapChangerControl",
    "TapChangerInfo",
    "TapSchedule",
    "TapeShieldCableInfo",
    "Terminal",
    "ThermostatControlMode",
    "ThermostatController",
    "TimeIntervalKind",
    "TopologicalIsland",
    "TopologicalNode",
    "TransformerCoreAdmittance",
    "TransformerEnd",
    "TransformerEndInfo",
    "TransformerMeshImpedance",
    "TransformerTank",
    "TransformerTankEnd",
    "TransformerTankInfo",
    "TransformerTest",
    "UnderfrequencyTripCurve",
    "UndervoltageTripCurve",
    "UnitMultiplier",
    "UnitSymbol",
    "UsagePoint",
    "UsagePointConnectedKind",
    "UsagePointGroup",
    "Version",
    "VoltVarCurve",
    "VoltVarSettings",
    "VoltWattCurve",
    "VoltWattSettings",
    "VoltageControlZone",
    "VoltageTripSettings",
    "WattVarCurve",
    "WattVarSettings",
    "WindingConnection",
    "WireAssemblyInfo",
    "WireInfo",
    "WireInsulationKind",
    "WireMaterialKind",
    "WirePhaseInfo",
    "WirePosition",
    "WireSpacingInfo",
]



from __future__ import annotations
from typing import Dict, List, Optional
import json

import cimlab.data_profile as cim

def add_to_catalog(obj: object, catalog: Dict) -> Dict:
    if obj.mRID == None:
        raise ValueError('Object must contain an mRID')
    if obj.mRID not in catalog:
        catalog[obj.mRID] = obj
    return catalog

def add_to_typed_catalog(obj: object, typed_catalog: Dict) -> Dict:
    if type(obj) not in typed_catalog:
        typed_catalog[type(obj)] = {}
    if obj.mRID not in typed_catalog[type(obj)]:
        typed_catalog[type(obj)][obj.mRID] = obj
    return typed_catalog


def get_all_by_type(typed_catalog: Dict, obj_type: type) -> List[object]:
    objects = typed_catalog.get(obj_type) if typed_catalog.get(obj_type) is not None else []
    for obj in objects:
        if obj.mRID not in self.is_loaded:
            load_all_attributes(obj)
            
def cim_dump(typed_catalog:Dict, cim_class:type):
    mrid_list = list(typed_catalog[cim_class].keys())
    attribute_list = list(cim_class().__dict__.keys())
    json_dump = {}

    for mrid in mrid_list:
        json_dump[mrid] = {}
        for attribute in attribute_list:
            value = getattr(typed_catalog[cim_class][mrid], attribute)
            json_dump[mrid][attribute] = item_dump(value)
    return json.dumps(json_dump)
                        
def item_dump(value):
    if type(value) is str:
        result = value
    elif type(value) is list:
        result = []
        for item in value:
            result.append(item_dump(item))
    elif value is None:
        result = ''
    elif type(type(value)) is type:
        result = value.mRID
    else:
        result = value
    return result


from SPARQLWrapper import SPARQLWrapper, JSON, POST
import re

from cimlab.loaders import ConnectionInterface, ConnectionParameters, Parameter, QueryResponse
from cimlab.loaders.blazegraph.blazegraph import BlazegraphConnection
# from cimlab.loaders.blazegraph.query_parsers import query_parser, query_list_parser



from __future__ import annotations
from typing import List, Dict, Optional
from dataclasses import dataclass, field
import cimlab.data_profile.rc4_2021 as cim
def get_all_attributes(feeder_mrid: str, typed_catalog: dict[type, dict[str, object]]) -> str: 
    """ Generates SPARQL query string for a given catalog of objects and feeder id
    Args:
        feeder_mrid (str | Feeder object): The mRID of the feeder or feeder object
        typed_catalog (dict[type, dict[str, object]]): The typed catalog of CIM objects organized by 
            class type and object mRID
    Returns:
        query_message: query string that can be used in blazegraph connection or STOMP client
    """

    mrid_list = list(typed_catalog[cim.Fuse].keys())
    
    query_message = """
        PREFIX r:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX cim:  <http://iec.ch/TC57/CIM100#>
        SELECT ?mRID ?name ?BaseVoltage ?Location ?ratedCurrent ?normalOpen ?open ?retained
        (group_concat(distinct ?Terminal; separator=";") as ?Terminals) 
        (group_concat(distinct ?Measurement; separator=";") as ?Measurements) 
        (group_concat(distinct ?Switch_Phase; separator=";") as ?SwitchPhase) 

        WHERE {          
          ?eq r:type cim:Fuse.
          VALUES ?fdrid {"%s"}
          VALUES ?mRID {"""%feeder_mrid
    # add all equipment mRID
    for mrid in mrid_list:
        query_message += ' "%s" \n'%mrid
    # add all attributes
    query_message += """               } 
        ?eq cim:Equipment.EquipmentContainer ?fdr.
        ?fdr cim:IdentifiedObject.mRID ?fdrid.
        ?eq cim:IdentifiedObject.mRID ?mRID.
        ?eq cim:IdentifiedObject.name ?name.
        
        OPTIONAL {?eq cim:ConductingEquipment.BaseVoltage ?bv.
        ?bv cim:IdentifiedObject.mRID ?BaseVoltage.}

        OPTIONAL {?eq cim:PowerSystemResource.Location ?loc.
        ?loc cim:IdentifiedObject.mRID ?Location.}

        ?t cim:Terminal.ConductingEquipment ?eq.
        ?t cim:IdentifiedObject.mRID ?Terminal

        OPTIONAL {?meas cim:Measurement.PowerSystemResource ?eq.
          ?meas cim:IdentifiedObject.mRID ?Measurement}

        OPTIONAL {?eq cim:Switch.ratedCurrent ?ratedCurrent.}
        OPTIONAL {?eq cim:Switch.normalOpen ?normalOpen.}
        OPTIONAL {?eq cim:Switch.open ?open.}
        OPTIONAL {?eq cim:Switch.retained ?retained.}

        OPTIONAL {?phs cim:SwitchPhase.Switch ?eq.
                  ?phs cim:IdentifiedObject ?Switch_Phase.}
        }
        GROUP by ?mRID ?name ?BaseVoltage ?Location ?ratedCurrent ?normalOpen ?open ?retained

        ORDER by  ?name
        """
    return query_message